package com.example.module;

import static android.view.View.VISIBLE;

import android.annotation.SuppressLint;
import android.content.res.Configuration;
import android.os.Bundle;
import android.content.Intent;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentActivity;

import com.unity3d.player.IUnityPlayerLifecycleEvents;
import com.unity3d.player.MultiWindowSupport;
import com.unity3d.player.UnityPlayer;

import io.flutter.embedding.android.FlutterFragment;
import io.flutter.embedding.android.RenderMode;
import io.flutter.embedding.android.TransparencyMode;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.FlutterEngineCache;
import io.flutter.embedding.engine.dart.DartExecutor;
import io.flutter.plugins.GeneratedPluginRegistrant;

public class MainActivity extends FragmentActivity implements IUnityPlayerLifecycleEvents {

    private static final String FLUTTER_ENGINE_ID = "engine_1";
    private static final String TAG_FLUTTER_FRAGMENT = "flutter_module";

    private FlutterEngine flutterEngine;
    private FlutterFragment flutterFragment;

    protected UnityPlayer mUnityPlayer; // don't change the name of this variable; referenced from native code

    // Setup activity layout
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // Unity初期化
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        super.onCreate(savedInstanceState);

        String cmdLine = updateUnityCommandLineArguments(getIntent().getStringExtra("unity"));
        getIntent().putExtra("unity", cmdLine);

        var rootViewId = View.generateViewId();

        mUnityPlayer = new UnityPlayer(this, this);
        mUnityPlayer.setId(rootViewId);
        setContentView(mUnityPlayer);
        mUnityPlayer.requestFocus();

        // FlutterEngineのインスタンスを生成
        flutterEngine = new FlutterEngine(this);

        // FlutterFragment生成時にCacheEngineを使用するのでCacheに登録しておく
        FlutterEngineCache.getInstance().put(FLUTTER_ENGINE_ID, flutterEngine);

        // Flutter側のエントリポイントを指定。default = main()
        flutterEngine.getDartExecutor().executeDartEntrypoint(DartExecutor.DartEntrypoint.createDefault());

        // FlutterのPlugin管理のためEngineをセット
        GeneratedPluginRegistrant.registerWith(flutterEngine);

        // CacheしてあるEngineを使用しあらかじめFlutterFragmentを作成しておく
        flutterFragment = FlutterFragment.withCachedEngine(FLUTTER_ENGINE_ID).renderMode(RenderMode.surface).transparencyMode(TransparencyMode.transparent).build();
        getSupportFragmentManager().beginTransaction().add(rootViewId, flutterFragment, TAG_FLUTTER_FRAGMENT).commit();
    }

    // Override this in your custom UnityPlayerActivity to tweak the command line arguments passed to the Unity Android Player
    // The command line arguments are passed as a string, separated by spaces
    // UnityPlayerActivity calls this from 'onCreate'
    // Supported: -force-gles20, -force-gles30, -force-gles31, -force-gles31aep, -force-gles32, -force-gles, -force-vulkan
    // See https://docs.unity3d.com/Manual/CommandLineArguments.html
    // @param cmdLine the current command line arguments, may be null
    // @return the modified command line string or null
    protected String updateUnityCommandLineArguments(String cmdLine) {
        return cmdLine;
    }

    // When Unity player unloaded move task to background
    @Override
    public void onUnityPlayerUnloaded() {
        moveTaskToBack(true);
    }

    // Callback before Unity player process is killed
    @Override
    public void onUnityPlayerQuitted() {
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        // To support deep linking, we need to make sure that the client can get access to
        // the last sent intent. The clients access this through a JNI api that allows them
        // to get the intent set on launch. To update that after launch we have to manually
        // replace the intent with the one caught here.
        mUnityPlayer.newIntent(intent);
        flutterFragment.onNewIntent(intent);
    }

    @Override
    protected void onStart() {
        super.onStart();

        if (!MultiWindowSupport.isInMultiWindowMode(this))
            return;

        mUnityPlayer.resume();
    }

    // If the activity is in multi window mode or resizing the activity is allowed we will use
    // onStart/onStop (the visibility callbacks) to determine when to pause/resume.
    // Otherwise it will be done in onPause/onResume as Unity has done historically to preserve
    // existing behavior.
    @Override
    protected void onStop() {
        super.onStop();

        if (!MultiWindowSupport.isInMultiWindowMode(this))
            return;

        mUnityPlayer.pause();
    }

    // Quit Unity
    @Override
    protected void onDestroy() {
        mUnityPlayer.destroy();

        getSupportFragmentManager().beginTransaction().remove(flutterFragment).commit();
        flutterEngine.getLifecycleChannel().appIsDetached();
        flutterEngine.destroy();

        super.onDestroy();
    }

    // Pause Unity
    @Override
    protected void onPause() {
        super.onPause();

        flutterFragment.onPause();
        flutterEngine.getLifecycleChannel().appIsPaused();

        MultiWindowSupport.saveMultiWindowMode(this);

        if (MultiWindowSupport.isInMultiWindowMode(this))
            return;

        mUnityPlayer.pause();
    }

    // Resume Unity
    @Override
    protected void onResume() {
        super.onResume();

        flutterFragment.onResume();
        flutterEngine.getLifecycleChannel().appIsResumed();

        if (MultiWindowSupport.isInMultiWindowMode(this) && !MultiWindowSupport.isMultiWindowModeChangedToTrue(this))
            return;

        mUnityPlayer.resume();
    }

    @Override
    public void onPostResume() {
        super.onPostResume();
        flutterFragment.onPostResume();
    }

    // Low Memory Unity
    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mUnityPlayer.lowMemory();
    }

    // Trim Memory Unity
    @Override
    public void onTrimMemory(int level) {
        super.onTrimMemory(level);
        if (level == TRIM_MEMORY_RUNNING_CRITICAL) {
            mUnityPlayer.lowMemory();
        }
    }

    // This ensures the layout will be correct.
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        flutterFragment.onConfigurationChanged(newConfig);
        mUnityPlayer.configurationChanged(newConfig);
    }

    // Notify Unity of the focus change.
    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        mUnityPlayer.windowFocusChanged(hasFocus);
    }

    @SuppressLint("RestrictedApi")
    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_MULTIPLE) {
            return mUnityPlayer.injectEvent(event);
        }
        return super.dispatchKeyEvent(event);
    }

    // Pass any events not handled by (unfocused) views straight to UnityPlayer
    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        // Flutterが表示されていればBackKey通知をFlutterで処理する
        if(event.getKeyCode() == KeyEvent.KEYCODE_BACK) {
            if(flutterFragment.getView().getVisibility() == VISIBLE) {
                flutterFragment.onBackPressed();
                return true;
            }
        }
        return mUnityPlayer.onKeyUp(keyCode, event);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // Flutterが表示されていればBackKey通知をFlutterで処理する
        if(event.getKeyCode() == KeyEvent.KEYCODE_BACK) {
            if(flutterFragment.getView().getVisibility() == VISIBLE) {
                return true;
            }
        }
        return mUnityPlayer.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        return mUnityPlayer.onTouchEvent(event);
    }

    @Override
    public boolean onGenericMotionEvent(MotionEvent event) {
        return mUnityPlayer.onGenericMotionEvent(event);
    }

    @Override
    public void onRequestPermissionsResult(
            int requestCode,
            @NonNull String[] permissions,
            @NonNull int[] grantResults
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        flutterFragment.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    public void onActivityResult(
            int requestCode,
            int resultCode,
            @Nullable Intent data
    ) {
        super.onActivityResult(requestCode, resultCode, data);
        flutterFragment.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onUserLeaveHint() {
        super.onUserLeaveHint();
        flutterFragment.onUserLeaveHint();
    }
}
